@@ -311,8 +311,8 @@ class TestMakeInfoFiles(misc.StdoutAssertionsMixin,
 
         # call _makeInfoFiles() and check that correct exception is raised
         with self.assertRaisesRegex(create_worker.CreateWorkerError,
-                                    "error creating directory %s: err-msg" %
-                                    _regexp_path("bdir", "info")):
+                                    "error creating directory {}: err-msg".format(
+                                            _regexp_path("bdir", "info"))):
             create_worker._makeInfoFiles("bdir", quiet)
 
         # check output to stdout
@@ -368,8 +368,8 @@ class TestMakeInfoFiles(misc.StdoutAssertionsMixin,
             self.assertWasQuiet()
         else:
             self.assertStdoutEqual(
-                "Creating %s, you need to edit it appropriately.\n" %
-                os.path.join("info", "admin"))
+                "Creating {}, you need to edit it appropriately.\n".format(
+                        os.path.join("info", "admin")))
 
     def testOpenError(self):
         """
@@ -432,15 +432,15 @@ class TestMakeInfoFiles(misc.StdoutAssertionsMixin,
             self.assertWasQuiet()
         else:
             self.assertStdoutEqual(
-                "mkdir %s\n"
-                "Creating %s, you need to edit it appropriately.\n"
-                "Creating %s, you need to edit it appropriately.\n"
-                "Not creating %s - add it if you wish\n"
-                "Please edit the files in %s appropriately.\n" %
-                (info_path, os.path.join("info", "admin"),
-                 os.path.join("info", "host"),
-                 os.path.join("info", "access_uri"),
-                 info_path))
+                ("mkdir {}\n"
+                 "Creating {}, you need to edit it appropriately.\n"
+                 "Creating {}, you need to edit it appropriately.\n"
+                 "Not creating {} - add it if you wish\n"
+                 "Please edit the files in {} appropriately.\n").format(
+                info_path, os.path.join("info", "admin"),
+                os.path.join("info", "host"),
+                os.path.join("info", "access_uri"),
+                info_path))
 
     def testCreatedSuccessfully(self):
         """
@@ -479,6 +479,8 @@ class TestCreateWorker(misc.StdoutAssertionsMixin, unittest.TestCase):
         "no-logrotate": False,
         "relocatable": False,
         "quiet": False,
+        "use-tls": False,
+        "delete-leftover-dirs": False,
         # options
         "basedir": "bdir",
         "allow-shutdown": None,
@@ -488,7 +490,9 @@ class TestCreateWorker(misc.StdoutAssertionsMixin, unittest.TestCase):
         "keepalive": 4,
         "maxdelay": 2,
         "numcpus": None,
+        "protocol": "pb",
         "maxretries": None,
+        "proxy-connection-string": None,
 
         # arguments
         "host": "masterhost",
@@ -575,6 +579,7 @@ class TestCreateWorker(misc.StdoutAssertionsMixin, unittest.TestCase):
         script and does typical for TAC file logic.
         """
 
+        # pylint: disable=import-outside-toplevel
         # import modules for mocking
         import twisted.application.service
         import twisted.python.logfile
@@ -602,7 +607,7 @@ class TestCreateWorker(misc.StdoutAssertionsMixin, unittest.TestCase):
         # Executed .tac file with mocked functions with side effect.
         # This will raise exception if .tac file is not valid Python file.
         glb = {}
-        exec(expected_tac_contents, glb, glb)
+        exec(expected_tac_contents, glb, glb)  # pylint: disable=exec-used
 
         # only one Application must be created in .tac
         application_class_mock.assert_called_once_with("buildbot-worker")
@@ -617,9 +622,13 @@ class TestCreateWorker(misc.StdoutAssertionsMixin, unittest.TestCase):
             options["keepalive"],
             umask=options["umask"],
             numcpus=options["numcpus"],
+            protocol=options["protocol"],
             maxdelay=options["maxdelay"],
             allow_shutdown=options["allow-shutdown"],
             maxRetries=options["maxretries"],
+            useTls=options["use-tls"],
+            delete_leftover_dirs=options["delete-leftover-dirs"],
+            proxy_connection_string=options["proxy-connection-string"],
             )
 
         # check that Worker instance attached to application
@@ -725,6 +734,30 @@ class TestCreateWorker(misc.StdoutAssertionsMixin, unittest.TestCase):
         # check that correct info message was printed
         self.assertStdoutEqual("worker configured in bdir\n")
 
+    def testUseTLS(self):
+        """
+        test that when --use-tls options is used, correct connection_string
+        is generated
+        """
+        options = self.options.copy()
+        options["use-tls"] = True
+
+        # patch _make*() functions to do nothing
+        self.setUpMakeFunctions()
+
+        # call createWorker() and check that we get success exit code
+        self.assertEqual(create_worker.createWorker(options), 0,
+                         "unexpected exit code")
+
+        # check _make*() functions were called with correct arguments
+        expected_tac_contents = ("".join(create_worker.workerTACTemplate)) % options
+        self.assertMakeFunctionsCalls(self.options["basedir"],
+                                      expected_tac_contents,
+                                      self.options["quiet"])
+
+        # check that correct info message was printed
+        self.assertStdoutEqual("worker configured in bdir\n")
+
     def testWithOpts(self):
         """
         test calling createWorker() with --relocatable and --allow-shutdown
@@ -775,3 +808,26 @@ class TestCreateWorker(misc.StdoutAssertionsMixin, unittest.TestCase):
 
         # there should be no output on stdout
         self.assertWasQuiet()
+
+    def testDeleteLeftoverDirs(self):
+        """
+        test calling createWorker() with --delete-leftover-dirs flag
+        """
+        options = self.options.copy()
+        options["delete-leftover-dirs"] = True
+
+        # patch _make*() functions to do nothing
+        self.setUpMakeFunctions()
+
+        # call createWorker() and check that we get success exit code
+        self.assertEqual(create_worker.createWorker(options), 0,
+                         "unexpected exit code")
+
+        # check _make*() functions were called with correct arguments
+        expected_tac_contents = ("".join(create_worker.workerTACTemplate)) % options
+        self.assertMakeFunctionsCalls(self.options["basedir"],
+                                      expected_tac_contents,
+                                      self.options["quiet"])
+
+        # check that correct info message was printed
+        self.assertStdoutEqual("worker configured in bdir\n")
